/*
 * Copyright © 2022 Soren Stoutner <soren@stoutner.com>.
 *
 * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
 *
 * Privacy Browser PC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Privacy Browser PC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Privacy Browser PC.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef BROWSERVIEW_H
#define BROWSERVIEW_H

// KDE Framework headers.
#include <KLineEdit>

// Qt framework headers.
#include <QPushButton>
#include <QWebEngineHistory>
#include <QWebEngineSettings>
#include <QWebEngineView>

class BrowserView : public QWidget
{
    // Include the Q_OBJECT macro.
    Q_OBJECT

public:
    // The primary contructor.
    explicit BrowserView(QWidget *parent);

    // The public functions.
    void applyOnTheFlyZoomFactor(const double &zoomFactor);
    void loadInitialWebsite();

signals:
    // The signals.
    void hideProgressBar() const;
    void linkHovered(const QString &linkUrl) const;
    void showProgressBar(const int &progress) const;
    void updateBackAction(const bool &isEnabled) const;
    void updateDomainSettingsIndicator(const bool status) const;
    void updateForwardAction(const bool &isEnabled) const;
    void updateJavaScriptAction(const bool &isEnabled) const;
    void updateSearchEngineActions(const QString &searchEngine) const;
    void updateUrlLineEdit(const QString &newUrl) const;
    void updateUserAgentActions(const QString &userAgent) const;
    void updateZoomFactorAction(const double &zoomFactor) const;

public Q_SLOTS:
    // The public slots.
    void applyApplicationSettings();
    void applyDomainSettingsAndReload();
    void applyDomainSettingsWithoutReloading(const QString &hostname);
    void applyOnTheFlySearchEngine(QAction *searchEngineActionPointer);
    void applyOnTheFlyUserAgent(QAction *userAgentActionPointer) const;
    void back() const;
    void forward() const;
    void home() const;
    void loadUrlFromLineEdit(QString url) const;
    void toggleJavaScript() const;
    void refresh() const;

private Q_SLOTS:
    // The private slots.
    void loadFinished() const;
    void loadProgress(const int &progress) const;
    void loadStarted() const;
    void pageLinkHovered(const QString &linkUrl) const;
    void updateUrl(const QUrl &url) const;

private:
    // The private variables.
    double currentZoomFactor;  // This can be removed once <https://redmine.stoutner.com/issues/799> has been resolved.
    QString searchEngineUrl;
    QWebEngineHistory *webEngineHistoryPointer;
    QWebEngineProfile *webEngineProfilePointer;
    QWebEngineSettings *webEngineSettingsPointer;
    QWebEngineView *webEngineViewPointer;

    // The private functions.
    void applyDomainSettings(const QString &hostname, const bool reloadWebsite);
};
#endif
